/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, S. Chassande-Barrioz, A. Lefebvre
 */

package org.objectweb.medor.optim.jorm;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.query.TestQueryTreeHelper;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.lib.QueryTreePrinter;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Collection;

/**
 * @author S.Chassande-Barrioz
 */
public abstract class TestJormLeafRewriter extends TestJormPDHelper {

	public TestJormLeafRewriter(String testName, String loggerName) {
		super(testName, loggerName);
	}

	protected abstract QueryLeaf getQueryLeaf_A() throws MedorException;

	protected abstract QueryLeaf getQueryLeaf_One() throws MedorException;

	public Example getExtent(Collection fileNames,
							 String className,
							 String pnameFieldName,
							 String[] fieldNames) {
		Example e = super.getExtent(fileNames, className, pnameFieldName, fieldNames);
		e.extent.setPMapper(mapper, PROJECT_NAME);
		return e;
	}

	public void testA() {
		checkRewrittenA(getRewrittenA());
	}

	public void testOne() {
		checkRewrittenOne(getRewrittenOne());
	}

	public void testB() {
		checkRewrittenB(getRewrittenB());
	}

	public void testC() {
		checkRewrittenC(getRewrittenC());
	}

	public void testD() {
		checkRewrittenD(getRewrittenD());
	}

	public void checkRewrittenA(Example e) {
		try {
			String prefix = e.clazz.getFQName() + ".";
			QueryLeaf ql = getQueryLeaf_A();
			QueryTreeField id = (QueryTreeField)
				ql.getTupleStructure().getField(prefix + "id");
			QueryTreeField f1 = (QueryTreeField)
				ql.getTupleStructure().getField(prefix + "field1");
			QueryTreeField f2 = (QueryTreeField)
				ql.getTupleStructure().getField(prefix + "field2");

			SelectProject expected = new SelectProject("", null);
			SinglePName spn = new SinglePName(
				new BasicFieldOperand(id),
				new BasicParameterOperand(
					PTypeSpaceMedor.NAMING_CONTEXT, prefix + "object"),
				e.extent.getField(prefix + "object").getType());
			expected.addCalculatedField(prefix + "object", spn.getType(), spn);
			expected.addPropagatedField(
				prefix + "field1", f1.getType(), new QueryTreeField[]{f1});
			expected.addPropagatedField(
				prefix + "field2", f2.getType(), new QueryTreeField[]{f2});

			logger.log(BasicLevel.DEBUG, "Expected QueryTree");
			QueryTreePrinter.printQueryTree(expected, logger);

			TestQueryTreeHelper.equals("", expected, e.qt, logger);
		}
		catch (MedorException me) {
			me.printStackTrace();
			fail(me.getMessage());
		}
	}

	public void checkRewrittenOne(Example e) {
		try {
			QueryLeaf ql = getQueryLeaf_One();
			String prefix = e.clazz.getFQName() + ".";
			QueryTreeField id = (QueryTreeField)
				ql.getTupleStructure().getField(prefix + "id");
			QueryTreeField f1 = (QueryTreeField)
				ql.getTupleStructure().getField(prefix + "field1");
			QueryTreeField ida = (QueryTreeField)
				ql.getTupleStructure().getField(prefix + "idA");

			SelectProject expected = new SelectProject("", null);
			SinglePName spn = new SinglePName(
				new BasicFieldOperand(id),
				new BasicParameterOperand(
					PTypeSpaceMedor.NAMING_CONTEXT, prefix + "." + "object"),
				e.extent.getField(prefix + "object").getType());
			expected.addCalculatedField(prefix + "object", spn.getType(), spn);
			expected.addPropagatedField(
				prefix + "field1", f1.getType(), new QueryTreeField[]{f1});
			String poName = prefix + "fieldA";
			spn = new SinglePName(
				new BasicFieldOperand(id),
				new BasicParameterOperand(
					PTypeSpaceMedor.NAMING_CONTEXT, poName),
				e.extent.getField(prefix + "fieldA").getType());
			expected.addCalculatedField(prefix + "fieldA", spn.getType(), spn);

			logger.log(BasicLevel.DEBUG, "Expected QueryTree");
			QueryTreePrinter.printQueryTree(expected, logger);

			TestQueryTreeHelper.equals("", expected, e.qt, logger);
		}
		catch (MedorException me) {
			me.printStackTrace();
			fail(me.getMessage());
		}
	}

	public void checkRewrittenB(Example e) {
//TODO
	}

	public void checkRewrittenC(Example e) {
//TODO
	}

	public void checkRewrittenD(Example e) {
//TODO
	}
}
