/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, S. Chassande-Barrioz, A. Lefebvre
 */
package org.objectweb.medor.optim.jorm;

import org.objectweb.medor.jorm.TestJormHelper;

import java.util.ArrayList;

/**
 * @author S.Chassande-Barrioz
 */
public abstract class TestJormPDHelper extends TestJormHelper {


    public TestJormPDHelper(String testName, String loggerName) {
        super(testName, loggerName);
    }

    public Example getExtentOfA() {
        ArrayList al = new ArrayList();
        al.add("org/objectweb/medor/optim/jorm/rdb/A.pd");
        return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.A");
    }

    public Example getExtentOfAgg() {
        ArrayList al = new ArrayList();
        al.add("org/objectweb/medor/optim/jorm/rdb/Agg.pd");
        return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.Agg");
    }

    public Example getExtentOfOne() {
        ArrayList al = new ArrayList();
        al.add("org/objectweb/medor/optim/jorm/rdb/One.pd");
        return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.One");
    }

	public Example getExtentOfB() {
		ArrayList al = new ArrayList();
		al.add("org/objectweb/medor/optim/jorm/rdb/ByteCharCN.pd");
		al.add("org/objectweb/medor/optim/jorm/rdb/B.pd");
		return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.B");
	}

	public Example getExtentOfB(String[] fieldnames) {
		ArrayList al = new ArrayList();
		al.add("org/objectweb/medor/optim/jorm/rdb/ByteCharCN.pd");
		al.add("org/objectweb/medor/optim/jorm/rdb/B.pd");
		return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.B",
				DEFAULT_PNAME_FIELD_NAME, fieldnames);
	}

    public Example getExtentOfC() {
        ArrayList al = new ArrayList();
        al.add("org/objectweb/medor/optim/jorm/rdb/C.pd");
        return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.C");
    }

    public Example getExtentOfD() {
        ArrayList al = new ArrayList();
        al.add("org/objectweb/medor/optim/jorm/rdb/ByteCharCN.pd");
        al.add("org/objectweb/medor/optim/jorm/rdb/D.pd");
        return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.D");
    }

	public Example getExtentOfAcolocB() {
		 ArrayList al = new ArrayList();
		 al.add("org/objectweb/medor/optim/jorm/rdb/AcolocB.pd");
		 return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.AcolocB");
	 }

	 public Example getExtentOfBcolocC() {
		 ArrayList al = new ArrayList();
		 al.add("org/objectweb/medor/optim/jorm/rdb/BcolocC.pd");
		 return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.BcolocC");
	 }

	public Example getExtentOfCcoloc() {
		ArrayList al = new ArrayList();
		al.add("org/objectweb/medor/optim/jorm/rdb/Ccoloc.pd");
		return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.Ccoloc");
	}

	public Example getExtentOfCustomer() {
		ArrayList al = new ArrayList();
		al.add("org/objectweb/medor/optim/jorm/rdb/Customer.pd");
		return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.Customer");
	}

	public Example getExtentOfCruise() {
		ArrayList al = new ArrayList();
		al.add("org/objectweb/medor/optim/jorm/rdb/Cruise.pd");
		return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.Cruise");
	}

	public Example getExtentOfRes() {
		ArrayList al = new ArrayList();
		al.add("org/objectweb/medor/optim/jorm/rdb/Res.pd");
		return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.Res");
	}

	public Example getExtentOfAddress() {
		ArrayList al = new ArrayList();
		al.add("org/objectweb/medor/optim/jorm/rdb/Jt2_Address.pd");
		return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.Jt2_Address");
	}

	public Example getExtentOfAnimal() {
		ArrayList al = new ArrayList();
		al.add("org/objectweb/medor/optim/jorm/rdb/Animal.pd");
		al.add("org/objectweb/medor/optim/jorm/rdb/Koala.pd");
		al.add("org/objectweb/medor/optim/jorm/rdb/Kangaroo.pd");
		return getExtent(al, "org.objectweb.medor.optim.jorm.rdb.Animal");
	}

    public Example getRewrittenA() {
        return getRewritten(getExtentOfA());
    }

    public Example getRewrittenOne() {
        return getRewritten(getExtentOfOne());
    }

    public Example getRewrittenB() {
        return getRewritten(getExtentOfB());
    }

    public Example getRewrittenC() {
        return getRewritten(getExtentOfC());
    }

    public Example getRewrittenD() {
        return getRewritten(getExtentOfD());
    }
}
