/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, S. Chassande-Barrioz, A. Lefebvre
 */
package org.objectweb.medor.optim.jorm.rdb;

import junit.framework.Test;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.mapper.rdb.lib.MapperJDBC;
import org.objectweb.jorm.mapper.rdb.lib.ConnectionSpecJDBC;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.lib.Log;
import org.objectweb.medor.optim.api.QueryRewriter;
import org.objectweb.medor.optim.jorm.JormFlatten2Rdb;
import org.objectweb.medor.optim.jorm.TestJormLeafRewriter;
import org.objectweb.medor.optim.lib.BasicQueryRewriter;
import org.objectweb.medor.optim.lib.FlattenQueryTreeRule;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.rdb.api.QualifiedTable;
import org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf;
import org.objectweb.medor.query.rdb.lib.BasicQualifiedTable;
import org.objectweb.medor.query.rdb.lib.BasicRdbExpQueryLeaf;

import java.util.ArrayList;

/**
 * @author S.Chassande-Barrioz
 */
public class TestRdbJormLeafRewriter extends TestJormLeafRewriter {

	/**
	 * main method to launch the tests manually
	 */
	public static void main(String[] args) {
		TestRunner.run(suite());
	}

	/**
	 * This method creates a TestSuite object with the current tests
	 */
	public static Test suite() {
		return new TestSuite(TestRdbJormLeafRewriter.class);
	}

	public TestRdbJormLeafRewriter() {
		super("TestRdbJormLeafRewriter",
			Log.MEDOR_PREFIX + ".optim.jorm.rdb.leafrewriter");
	}

	public TestRdbJormLeafRewriter(String testName) {
		super(testName, Log.MEDOR_PREFIX + ".optim.jorm.rdb.leafrewriter");
	}

   public PMapper newMappper() {
        MapperJDBC m = null;
        try {
            m = new MapperJDBC();
			ConnectionSpecJDBC cs =
				new ConnectionSpecJDBC("jdbc:product:basename", "java.lang.String");
             m.setConnectionFactory(cs);
        }
        catch (PException e) {
            e.printStackTrace();
        }
        return m;
    }

	public QueryRewriter createQueryRewriter() {
		ArrayList rules = new ArrayList(2);
		rules.add(new FlattenQueryTreeRule());
		rules.add(new JormFlatten2Rdb());
		return new BasicQueryRewriter(rules);
	}

    protected QueryLeaf getQueryLeaf_A() throws MedorException {
        QualifiedTable table = new BasicQualifiedTable("T_REF_SING_A", "T_REF_SING_A_0");
        String name = "org.objectweb.medor.optim.jorm.rdb.A";
        String prefix = name + ".";
        RdbExpQueryLeaf leaf = new BasicRdbExpQueryLeaf(null,
            new QualifiedTable[] {table}, name);
        QueryTreeField id = leaf.addRdbField(
             prefix + "id", PTypeSpace.LONG, "id_col", table);
        QueryTreeField f1 = leaf.addRdbField(
             prefix + "field1", PTypeSpace.STRING, "FIELD_1", table);
        QueryTreeField f2 = leaf.addRdbField(
             prefix + "field2", PTypeSpace.STRING, "FIELD_2", table);
        return leaf;
    }

    protected QueryLeaf getQueryLeaf_One() throws MedorException {
        QualifiedTable table = new BasicQualifiedTable("T_REF_SING_ONE", "T_REF_SING_ONE_0");
        String name = "org.objectweb.medor.optim.jorm.rdb.One";
        String prefix = name + ".";
        RdbExpQueryLeaf leaf = new BasicRdbExpQueryLeaf(null,
            new QualifiedTable[] {table}, name);
        QueryTreeField id = leaf.addRdbField(
            prefix + "id", PTypeSpace.STRING, "ID", table);
        QueryTreeField f1 = leaf.addRdbField(
            prefix + "field1", PTypeSpace.STRING, "FIELD_1", table);
        QueryTreeField ida = leaf.addRdbField(
            prefix + "idA", PTypeSpace.LONG, "ID_A", table);
        return leaf;
    }

}
